package com.blazinggames.retro.classic.search;

// Java Imports
import java.awt.Point;
import java.util.Vector;
    
// Blazing Games Imports
import com.blazinggames.urcl.data.Config;
import com.blazinggames.urcl.ui.*;

// Retro Imports
import com.blazinggames.urcl.text.*;

/**
 * Ultimate Retro Episode 3: Classic Hide and Seek
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * The classic version of the Hide and Seek game. The mechanics of the game are very simple.
 * Both the player and the computer choose locations on a 2D grid and then take turns trying
 * to find the other. The first to located where their opponent is wins.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class HandS 
extends TextDisplay
{
    // *********************
    // ***** CONSTANTS *****
    // *********************
    
    private static final String VERSION = "1.10.031110";
    
    private static final String NUM_LEVELS = "03NUM_LEVELS";
    private static final String LEVEL="03LEVEL";
    private static final String INSTRUCTIONS = "03INSTRUCTIONS";
    

// *********************
// ***** VARIABLES *****
// *********************
    
    private String _levelNames[];
    private int _levelSizes[];
    private Instructions _instructions;

// **************************************    
// ***** METHODS - From TextDisplay *****
// **************************************

	/**
	 * initializes the game by reading the level information from the config file
	 */
    public boolean initialize(DisplayHandler dh)
    {
        super.initialize(dh);
        int n = _config.getIntProperty(NUM_LEVELS, 0);
        if (n < 1)
            n = 1;
        _levelNames = new String[n];
        _levelSizes = new int[n];
        String args[];
        for (int cntr = 0; cntr < n; ++cntr)
        {
            args = _config.parseString(_config.getStringProperty(LEVEL+"["+cntr+"]","unnamed,100,7"),',');
            if (args.length != 2)
            {
                _levelNames[cntr] = "Play the game";
                _levelSizes[cntr] = 100;
            }
            else
            {
                _levelNames[cntr] = args[0];
                _levelSizes[cntr] = _config.parseInt(args[1], 100);
            }
        }

        return true;
    }
    

    public void run()
    {
        String s;
        int choice;
        
        printHeader();
        do
        {
            println();
            println("Please choose:");
            println(" 0. Instructions");
            for (int cntr = 0; cntr < _levelNames.length; ++cntr)
            {
                println(" " + (cntr+1) + ". " + _levelNames[cntr]);
            }
            int nQuit = _levelNames.length + 1;
            if (_canQuit)
                println(" " + nQuit + ". Quit");
            print("Type number then return: ");
            s = input();
            choice = _config.parseInt(s, -1);
            println(s);
            println();
            if (choice == 0)
                instructions();
            else if ((choice > 0) && (choice < nQuit))
                game(_levelSizes[choice-1]);
            else if (choice == nQuit)
            {
                if (_canQuit)
                    _displayHandler.displayEvent(this, "SHOW 1");
                else
                    println("Quit command is unavailable");
            }
            else
            {
                println("You have made an invalid choice!");
                println("Please try again.");
            }
        }
        while(_active);
        _displayHandler.log("HideSeekDisplay run thread exiting");
    }
    
    private void printHeader()
    {
        println("*** Hide and Seek ***");
        println("Classic Version " + VERSION);
        println();
        println("Copyright (c) 2001, 2003");
        println("Blazing Games Inc.");
        println("All Rights Reserved.");
        println();
    }

    private void instructions()
    {
        if (_instructions == null)
        {
            println("Grabbing instructions...");
            _res.addTextGroup(INSTRUCTIONS, _config.getStringProperty(INSTRUCTIONS, "tank.txt"));
            _instructions = new Instructions(_res.getTextGroup(INSTRUCTIONS));
            println("Done.");
            println();
        }
        _instructions.showInstructions(this);
    }
    
    private void game(int size)
    {
        Point player = null;
        Point opponent = new Point(rand(size), rand(size));
        _displayHandler.log("Opponent at " + opponent.x +"," + opponent.y);
        Point playerGuess = null;
        Point oppGuess = new Point();
        Point oppLows = new Point(1,1);
        Point oppHighs = new Point(size, size);
        Vector oppGuessHistory = new Vector();
        boolean oppValidGuess;
        int cntr;
        
        println ("The grid is " + size +"x" + size);
        println("Where will you hide?");
        player = getCoords(size);
        println("You choose (" + player.x + "," + player.y + ")");
        println("The opponent has chosen it's location.");
        
        int round = 0;
        boolean gameOver = false;
        do
        {
            ++round;
            println("Round " + round);
            println("Your guess:");
            playerGuess = getCoords(size);
            println("You guessed (" + playerGuess.x + "," + playerGuess.y + ")");
            if (playerGuess.equals(opponent))
            {
                println("Well Done!");
                println("You found the opponent!");
                println();
                gameOver = true;
            }
            else
            {
                print("Opponent is");
                if (playerGuess.y < opponent.y)
                    print(" south");
                else if (playerGuess.y > opponent.y)
                    print(" north");
                if (playerGuess.x < opponent.x)
                    print(" east");
                else if (playerGuess.x > opponent.x)
                    print(" west");
                println(".");
            }
            
            if (!gameOver)
            {
                do
                {
                    oppGuess.x = rand(oppLows.x, oppHighs.x);
                    oppGuess.y = rand(oppLows.y, oppHighs.y);
                    oppValidGuess = true;
                    for (cntr = 0; cntr < oppGuessHistory.size(); ++cntr)
                    {
                        if ( oppGuess.equals((Point)oppGuessHistory.elementAt(cntr)) )
                            oppValidGuess = false;
                    }                
                }
                while (!oppValidGuess);
                oppGuessHistory.addElement(new Point(oppGuess));
                println("opponent guesses (" + oppGuess.x + "," + oppGuess.y + ")");
                
                if (oppGuess.equals(player))
                {
                    println("Your opponent found you!");
                    println("You lose!!!");
                    println("The opponent was at " + opponent.x + "," + opponent.y);
                    gameOver = true;
                }
                else
                {
                    if (oppGuess.x >= player.x)
                        oppHighs.x = oppGuess.x;
                    if (oppGuess.x <= player.x)
                        oppLows.x = oppGuess.x;
                    if (oppGuess.y >= player.y)
                        oppHighs.y = oppGuess.y;
                    if (oppGuess.y <= player.y)
                        oppLows.y = oppGuess.y;                
                }
            }
        }
        while (gameOver == false);
        
    }
    
    private Point getCoords(int size)
    {
        Point p = null;
        String s;
        String guess[];
        int tx, ty;
        
        do
        {
            s = input();
            guess = _config.parseString(s, ',');
            if (guess.length != 2)
            {
                tx = 0;
                ty = 0;
            }
            else
            {
                tx = _config.parseInt(guess[0].trim(), 0);
                ty = _config.parseInt(guess[1].trim(), 0);
            }
            
            if ( (tx<1) || (ty<1) || (tx>size) || (ty>size) )
            {
                println("When entering coordinates, please use");
                println("the format x,y and make sure the");
                println("numbers are between 1 and " + size);
                println();
                println("try again:");
            }
            else
            {
                p = new Point(tx, ty);
            }
        }
        while (p == null);
        return p;
    }
}
