package com.blazinggames.urcl.ui.playfield;

// Java imports
import java.awt.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * The base tilerendering class for use with the playfield class.
 * This tile renderer only renders solid colors.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class TileRenderer
{
// *********************
// ***** VARIABLES *****
// *********************

    protected Color _backfill;
    protected int _width;
    protected int _height;


// ************************
// ***** CONSTRUCTORs *****
// ************************

    /**
     * default constructor.
     * creates a black 32x32 tile
     */
    public TileRenderer()
    {
        _backfill = Color.black;
        _width = 32;
        _height = 32;
    }

    /**
     * size constructor.
     * creates a black tile of user specified size.
     *
     * @param w width of tile
     * @param h height of tile
     */
    public TileRenderer(int w, int h)
    {
        _backfill = Color.black;
        _width = w;
        _height = h;
    }

    /**
     * full constructor.
     *
     * @param w width of tile
     * @param h height of tile
     * @param c color of tile
     */
    public TileRenderer(int w, int h, Color c)
    {
        _backfill = c;
        _width = w;
        _height = h;
    }


// *******************************
// ***** METHODS - RENDERING *****
// *******************************

    /**
     * basic renderer. Draws tile at indicated coordinates.
     *
     * @param g graphics context to draw tile to
     * @param c component drawing occuring in
     * @param x x coordinate to draw tile
     * @param y y coordinate to draw tile
    */
    public void render(Graphics g, Component c, int x, int y)
    {
        g.setColor(_backfill);
        g.fillRect(x, y, _width, _height);
    }

    /**
     * state renderer. Draws tile at indicated coordinates using indicated state.
     * this renderer has no states so the state is ignored, but inherited
     * renderers may have multiple images based on state.
     *
     * @param g graphics context to draw tile to
     * @param c component drawing occuring in
     * @param x x coordinate to draw tile
     * @param y y coordinate to draw tile
     */
    public void render(Graphics g, Component c, int x, int y, int state)
    {
        g.setColor(_backfill);
        g.fillRect(x, y, _width, _height);
    }
    
}
